/*
* douluo-ui v1.0.3
* Copyright ©2022-2022 
* Released under the MIT License.
*/
'use strict';

//
var script$1 = {
  name: 'Render',
  functional: true,
  props: {
    scope: Object,
    render: Function
  },
  render: (h, ctx) => {
    return ctx.props.render ? ctx.props.render(h, ctx.props.scope) : '';
  }
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

/* script */
const __vue_script__$1 = script$1;

/* template */

  /* style */
  const __vue_inject_styles__$1 = undefined;
  /* scoped */
  const __vue_scope_id__$1 = undefined;
  /* module identifier */
  const __vue_module_identifier__$1 = undefined;
  /* functional template */
  const __vue_is_functional_template__$1 = undefined;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$1 = /*#__PURE__*/normalizeComponent(
    {},
    __vue_inject_styles__$1,
    __vue_script__$1,
    __vue_scope_id__$1,
    __vue_is_functional_template__$1,
    __vue_module_identifier__$1,
    false,
    undefined,
    undefined,
    undefined
  );

var forced = {
  selection: {
    renderHeader: (h, {
      store
    }) => {
      return h("el-checkbox", {
        "attrs": {
          "disabled": store.states.data && store.states.data.length === 0,
          "indeterminate": store.states.selection.length > 0 && !store.states.isAllSelected,
          "value": store.states.isAllSelected
        },
        "nativeOn": {
          "click": store.toggleAllSelection
        }
      });
    },
    renderCell: (h, {
      row,
      column,
      store,
      $index
    }) => {
      return h("el-checkbox", {
        "nativeOn": {
          "click": event => event.stopPropagation()
        },
        "attrs": {
          "value": store.isSelected(row),
          "disabled": column.selectable ? !column.selectable.call(null, row, $index) : false
        },
        "on": {
          "input": () => {
            store.commit('rowSelectedChanged', row);
          }
        }
      });
    },
    sortable: false,
    resizable: false
  },
  index: {
    renderHeader: (h, scope) => {
      return h("span", [scope.column.label || '#']);
    },
    renderCell: (h, {
      $index,
      column
    }) => {
      let i = $index + 1;
      const index = column.index;

      if (typeof index === 'number') {
        i = $index + index;
      } else if (typeof index === 'function') {
        i = index($index);
      }

      return h("div", [i]);
    },
    sortable: false
  },
  expand: {
    renderHeader: (h, scope) => {
      return h("span", [scope.column.label || '']);
    },
    renderCell: (h, {
      row,
      store
    }, proxy) => {
      const expanded = store.states.expandRows.indexOf(row) > -1;
      return h("div", {
        "class": 'el-table__expand-icon ' + (expanded ? 'el-table__expand-icon--expanded' : ''),
        "on": {
          "click": e => proxy.handleExpandClick(row, e)
        }
      }, [h("i", {
        "class": 'el-icon el-icon-arrow-right'
      })]);
    },
    sortable: false,
    resizable: false,
    className: 'el-table__expand-column'
  }
};

//
var script = {
  name: 'TableColumn',
  components: {
    RenderColumn: __vue_component__$1
  },
  props: {
    column: {
      type: Object,
      default: () => ({})
    },
    headerAlign: String,
    align: String,
    label: String,
    prop: String,
    width: [String, Number]
  },
  watch: {
    column: {
      handler() {
        this.setColumn();
      },

      immediate: true
    }
  },
  methods: {
    setColumn() {
      if (this.column.type) {
        this.column.renderHeader = forced[this.column.type].renderHeader;
        this.column.render = this.column.render || forced[this.column.type].renderCell;
      }

      if (this.column.formatter) {
        this.column.render = (h, scope) => {
          return h('span', null, scope.column.formatter(scope.row, scope.column, scope.row, scope.$index)); // return <span>{scope.column.formatter(scope.row, scope.column, scope.row, scope.$index)}</span>
        };
      }

      if (!this.column.render) {
        this.column.render = (h, scope) => {
          return h('span', null, scope.row[scope.column.property]); // return <span>{scope.row[scope.column.property]}</span>
        };
      }
    }

  }
};

/* script */
const __vue_script__ = script;

/* template */
var __vue_render__ = function () {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "el-table-column",
    _vm._g(
      _vm._b(
        {
          attrs: {
            prop: _vm.column.prop || _vm.prop,
            label: _vm.column.label || _vm.label,
            type: _vm.column.type,
            index: _vm.column.index,
            "column-key": _vm.column.columnKey,
            width: _vm.column.width || _vm.width,
            "min-width": _vm.column.minWidth,
            fixed: _vm.column.fixed,
            "render-header": _vm.column.renderHeader,
            sortable: _vm.column.sortable || false,
            "sort-method": _vm.column.sortMethod,
            "sort-by": _vm.column.sortBy,
            "sort-orders": _vm.column.sortOrders,
            resizable: _vm.column.resizable || true,
            formatter: _vm.column.formatter,
            "show-overflow-tooltip": _vm.column.showOverflowTooltip || false,
            align: _vm.column.align || _vm.align || "center",
            "header-align":
              _vm.column.headerAlign ||
              _vm.headerAlign ||
              _vm.column.align ||
              _vm.align ||
              "center",
            "class-name": _vm.column.className,
            "label-class-name": _vm.column.labelClassName,
            selectable: _vm.column.selectable,
            "reserve-selection": _vm.column.reserveSelection || false,
            filters: _vm.column.filters,
            "filter-placement": _vm.column.filterPlacement,
            "filter-multiple": _vm.column.filterMultiple,
            "filter-method": _vm.column.filterMethod,
            "filtered-value": _vm.column.filteredValue,
          },
          scopedSlots: _vm._u([
            {
              key: "header",
              fn: function (scope) {
                return [
                  _vm.column.renderHeader
                    ? _c("render-column", {
                        attrs: {
                          scope: scope,
                          render: _vm.column.renderHeader,
                        },
                      })
                    : _c("span", [_vm._v(_vm._s(scope.column.label))]),
                ]
              },
            },
            {
              key: "default",
              fn: function (scope) {
                return [
                  _c("render-column", {
                    attrs: { scope: scope, render: _vm.column.render },
                  }),
                ]
              },
            },
          ]),
        },
        "el-table-column",
        _vm.$attrs,
        false
      ),
      _vm.$listeners
    ),
    [
      _vm._v(" "),
      _vm._v(" "),
      _vm.column.children
        ? _vm._l(_vm.column.children, function (col, index) {
            return _c("dl-table-column", { key: index, attrs: { column: col } })
          })
        : _vm._e(),
    ],
    2
  )
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

  /* style */
  const __vue_inject_styles__ = undefined;
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__ = /*#__PURE__*/normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    undefined,
    undefined,
    undefined
  );

let componentNamespace = 'dl'; // 组件前缀命名只能更新一次，防止重复执行修改组件注册出错

let isUpdated = false; // scss 命名不能通过 js 更改，在 SCSS 编译已经打包好了
function createNamespace(name, {
  prefix
}) {
  if (!isUpdated) {
    isUpdated = true;
    componentNamespace = prefix || componentNamespace; // 更改前缀
  }

  return componentNamespace + name.charAt(0).toUpperCase() + name.substr(1);
}

__vue_component__.install = (Vue, options = {}) => {
  Vue.component(createNamespace('TableColumn', {
    prefix: options.prefix
  }), __vue_component__);
};

module.exports = __vue_component__;
